@@ -8,32 +8,41 @@
 
 
 /*!
-Alternatives and extensions to the unstable `char.encode_utf8()` and `char.encode_utf16()`.
-
-[crates.io page](https://crates.io/crates/encode_unicode)
-[github repository](https://github.com/tormol/encode_unicode)
+Miscellaneous UTF-8 and UTF-16 types and methods.
 
 # Optional features:
-* **no_std**: Use `#[no_std]`; There are some differences:
+* `#![no_std]`-mode: There are a few differences:
   * `AsciiExt` doesn't exist, but `is_ascii()` is made available as an inherent impl.
   * `Error` doesn't exist, but `description()` is made available as an inherent impl.
+  * `Extend`/`FromIterator`-implementations for `String`/`Vec<u8>`/`Vec<u16>`.
   * There is no `io`, so `Utf8Iterator` doesn't implement `Read`.
-  * The iterators doesn't implement `Debug`.
-* **ascii**: Convert `Utf8Char` and `Utf16Char` to and from [ascii](https://tomprogrammer.github.io/rust-ascii/ascii/index.html)::[`AsciiChar`](https://tomprogrammer.github.io/rust-ascii/ascii/enum.AsciiChar.html).
-* **ascii_no_std**: You need to use this feature instead of both ascii and no_std.
-  This is because the ascii crate needs to know about `#[no_std]`, but the features are otherwize independent.
+
+  This feature is enabled by setting `default-features=false` in `Cargo.toml`:
+  `encode_unicode = {version="0.3", default-features=false}`
+* Integration with the [ascii](https://tomprogrammer.github.io/rust-ascii/ascii/index.html) crate:  
+  Convert `Utf8Char` and `Utf16Char` to and from
+  [ascii::`AsciiChar`](https://tomprogrammer.github.io/rust-ascii/ascii/enum.AsciiChar.html).
+
+The minimum supported version of Rust is 1.15,
+older versions might work, but also break in a minor update.
+
+[crates.io page](https://crates.io/crates/encode_unicode)  
+[githab repository](https://github.com/tormol/encode_unicode)
+
 */
 
 
-#![warn(missing_docs)]
+#![cfg_attr(feature="std", warn(missing_docs))] // Don't bother documenting std standins.
 
-#![cfg_attr(feature="no_std", no_std)]
+#![cfg_attr(not(feature="std"), no_std)]
 // either `cargo clippy` doesn't see theese, or I get a warning when I build.
 #![cfg_attr(feature="clippy", feature(plugin))]
 #![cfg_attr(feature="clippy", plugin(clippy))]
-#![cfg_attr(feature="clippy", allow(len_without_is_empty))]// UtfxChar is never empty
-#![cfg_attr(feature="clippy", allow(match_same_arms))]
 #![cfg_attr(feature="clippy", allow(derive_hash_xor_eq))]// tested
+#![cfg_attr(feature="clippy", allow(len_without_is_empty))]// UtfxChar is never empty
+#![cfg_attr(feature="clippy", allow(match_same_arms))]// looks better IMO
+#![cfg_attr(feature="clippy", allow(needless_return))]// `foo.bar(); foo` looks unfinished
+#![cfg_attr(feature="clippy", allow(redundant_closure))]// keep it explicit
 // precedence: I prefer spaces to parentheses, but it's nice to recheck.
 
 mod errors;
@@ -55,7 +64,8 @@ pub mod error {// keeping the public interface in one file
     //! Errors returned by various conversion methods in this crate.
     pub use utf8_char::FromStrError;
     pub use errors::{InvalidCodepoint};
-    pub use errors::{InvalidUtf8FirstByte,InvalidUtf8};
+    pub use errors::{InvalidUtf8};
+    pub use errors::{InvalidUtf8FirstByte,InvalidUtf16FirstUnit};
     pub use errors::{InvalidUtf8Slice,InvalidUtf16Slice};
     pub use errors::{InvalidUtf8Array,InvalidUtf16Tuple};
 }
