@@ -8,17 +8,15 @@
 
 use CharExt;
 use Utf16Char;
-#[cfg(not(feature="no_std"))]
 extern crate core;
-#[cfg(not(feature="no_std"))]
 use self::core::fmt;
 
-// Invalid values that says the field consumed or empty.
+// Invalid values that says the field is consumed or empty.
 const FIRST_USED: u16 = 0x_dc_00;
 const SECOND_USED: u16 = 0;
 
-/// Iterate over the units in an UTF-16 representation of a codepoint.
-#[derive(Clone,Copy)]
+/// Iterate over the units of the UTF-16 representation of a codepoint.
+#[derive(Clone)]
 pub struct Utf16Iterator {
     first: u16,
     second: u16,
@@ -54,10 +52,13 @@ impl ExactSizeIterator for Utf16Iterator {
         (if self.second == SECOND_USED {0} else {1})
     }
 }
-#[cfg(not(feature="no_std"))]
 impl fmt::Debug for Utf16Iterator {
     fn fmt(&self,  fmtr: &mut fmt::Formatter) -> fmt::Result {
-        let v: Vec<u16> = self.collect();
-        write!(fmtr, "{:?}", v)
+        let mut clone = self.clone();
+        match (clone.next(), clone.next()) {
+            (Some(one), None)  => write!(fmtr, "[{}]", one),
+            (Some(a), Some(b)) => write!(fmtr, "[{}, {}]", a, b),
+            (None,  _)         => write!(fmtr, "[]"),
+        }
     }
 }
